import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import QtGraphicalEffects 1.0

import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1

import QtQuick.Window 2.0

import QtWebKit 3.0
import QtWebKit.experimental 1.0
import "js/script.js" as Tab 

MainView {
    id: root 
    width: Screen.width; height: Screen.height;
    property string currentTab: ""
    property bool hasTabOpen: (tabModel.count !== 0) && (typeof(Tab.itemMap[currentTab]) !== "undefined")
    property bool mobileUserAgent: false  
    property bool readerMode: false
    property alias automaticOrientation: orientationHelper.automaticOrientation

    FontLoader { id: fontAwesome; source: "icons/fontawesome-webfont.ttf" }  

    Component.onCompleted: {
        var db = getDatabase(); 
        if (Tab.ReopenPreviousTab) {
            db.transaction(
                function(tx) {
                    var result = tx.executeSql("SELECT * FROM previous"); 
                    for (var i=0; i < result.rows.length; i++) {
                        // FIXME: favicon doesn't display when re-open 
                        console.log('open previous closed page: ' + result.rows.item(i).url)
                        openNewTab('page-'+salt(), result.rows.item(i).url)
                    }
                    // FIXME: should match opened urls, not just drop 
                    tx.executeSql("DROP TABLE IF EXISTS previous");
                }
            );
        }
        bounce.start()
    }
    Component.onDestruction: { 
        if (Tab.ReopenPreviousTab) {
            var db = getDatabase();
            for (var openedUrl in Tab.itemMap) {
                db.transaction(
                    function(tx) { 
                        tx.executeSql('insert into previous values (?);',[Tab.itemMap[openedUrl].url]);
                        console.log('saving existing page: ' + Tab.itemMap[openedUrl].url)
                    }
                );
            }
        }
    }

    SequentialAnimation { 
        id: bounce 
        PropertyAnimation { target: container; properties: "anchors.leftMargin"; to: Tab.DrawerWidth; duration: 200; easing.type: Easing.InOutQuad; }
        PropertyAnimation { target: container; properties: "anchors.leftMargin"; to: "0"; duration: 400; easing.type: Easing.InOutQuad; }
    } 
    function openNewTab(pageid, url) {
        //console.log("openNewTab: "+ pageid + ', currentTab: ' + currentTab);
        if (hasTabOpen) {      
            tabModel.insert(0, { "title": "Loading..", "url": url, "pageid": pageid, "favicon": "icons/favicon.png" } );
            // hide current tab and display the new
            Tab.itemMap[currentTab].visible = false;
        } else {
            tabModel.set(0, { "title": "Loading..", "url": url, "pageid": pageid, "favicon": "icons/favicon.png" } );
        }
        var webView = tabView.createObject(container, { id: pageid, objectName: pageid } );
        webView.url = url; // FIXME: should use loadUrl() wrapper 

        Tab.itemMap[pageid] = webView;
        currentTab = pageid;
        tabListView.currentIndex = 0 // move highlight to top  
    }

    function switchToTab(pageid) {
        //console.log("switchToTab: "+ pageid + " , from: " + currentTab + ' , at ' + tabListView.currentIndex);
        if (currentTab !== pageid ) { 
            Tab.itemMap[currentTab].visible = false;
            currentTab = pageid;
        }
        Tab.itemMap[currentTab].visible = true;
        // assign url to text bar
        urlText.text = Tab.itemMap[currentTab].url;
    }

    function closeTab(deleteIndex, pageid) { 
        //console.log('closeTab: ' + pageid + ' at ' + deleteIndex + ': ' + tabModel.get(deleteIndex))
        //console.log('\ttabListView.model.get(deleteIndex): ' + tabListView.model.get(deleteIndex).pageid)
        Tab.itemMap[pageid].visible = false; 
        tabModel.remove(deleteIndex);
        Tab.itemMap[pageid].destroy(); 
        delete(Tab.itemMap[pageid])

        if (hasTabOpen) { 
            // FIXME: after closed, Qt 5.1 doesn't change tabListView.currentIndex  
            if (tabModel.count == 1 && tabListView.currentIndex == 1) tabListView.currentIndex = 0;  
            currentTab = tabListView.model.get( tabListView.currentIndex ).pageid
            switchToTab(currentTab)
        } else {
            urlText.text = "";
            currentTab = ""; // clean currentTab 
        }
    } 

    function salt(){
        var salt = ""
        for( var i=0; i < 5; i++ ) {
            salt += Tab.RandomString.charAt(Math.floor(Math.random() * Tab.RandomString.length));
        }
        return salt
    }

    function fixUrl(url) {
        url = url.replace( /^\s+/, "");
        url = url.replace( /\s+$/, "")
        url = url.replace( /(<([^>]+)>)/ig, ""); // remove <b> tag 
        if (url == "") return url;
        if (url[0] == '/') {
            var str = Tab.itemMap[currentTab].url.toString();
            var n = str.indexOf('/', 8); // search after http:// 
            var actualUrl = str.substring(0, n)+'/'+url.substring(1);
            return actualUrl
        }
        if (url[0] == '.') { 
            var str = Tab.itemMap[currentTab].url.toString();
            var n = str.lastIndexOf('/');
            var actualUrl = str.substring(0, n)+'/'+url.substring(1);
            return actualUrl
        }
        //FIXME: search engine support here
        if (url.indexOf('.') < 0) { return "http://www.google.com/search?q="+url; }
        if (url.indexOf(":")<0) { return "http://"+url; }
        else { return url;}
    }

    function loadUrl(url) {
        if (hasTabOpen) {
            Tab.itemMap[currentTab].url = fixUrl(url)
        } else { 
            openNewTab("page"+salt(), fixUrl(url));
        }
        Tab.itemMap[currentTab].focus = true;
    }
    function getDatabase() {
        var db = LocalStorage.openDatabaseSync("shellbrowser", "0.1", "history db", 100000);
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS history (url TEXT, title TEXT, icon TEXT, date INTEGER)');
            }
        );
        db.transaction(function(tx) {tx.executeSql('CREATE TABLE IF NOT EXISTS previous (url TEXT)'); });
        return db;
    }
    function updateHistory(url, title, icon) { 
        var date = new Date();
        var db = getDatabase();
        db.transaction(
            function(tx) {
                var result = tx.executeSql('delete from history where url=(?);',[url])
            }
        );
        db.transaction(
            function(tx) {
                var result = tx.executeSql('insert into history values (?,?,?,?);',[url, title, icon, date.getTime()])
                if (result.rowsAffected < 1) {
                    console.log("Error inserting url: " + url)
                } else {
                }
            }
        );
    }

    function highlightTerms(text, terms) {
        if (text === undefined || text === '') {
            return ''
        }
        var highlighted = text.toString()
        highlighted = highlighted.replace(new RegExp(terms, 'ig'), '<b>$&</b>')
        return highlighted
    }

    function queryHistory(str) {
        var db = getDatabase();
        var result; 
        db.transaction(
            function(tx) {
                result = tx.executeSql("select * from history where url like ?", ['%'+str+'%']) 
            }
        );
        historyModel.clear();
        historyListView.currentIndex = 0;
        for (var i=0; i < result.rows.length; i++) {
            historyModel.insert(0, {"url": highlightTerms(result.rows.item(i).url, str), 
            "title": result.rows.item(i).title});
        }
        historyListView.currentIndex = 0;
    }

    function toggleReaderMode() {
        if (readerMode) {
            Tab.itemMap[currentTab].reload();
        } else { 
            // FIXME: dirty hack to load js from local file 
            var xhr = new XMLHttpRequest;
            xhr.open("GET", "./js/readability.js");
            xhr.onreadystatechange = function() {
                if (xhr.readyState == XMLHttpRequest.DONE) {
                    var read = new Object({'type':'readability', 'content': xhr.responseText });
                    Tab.itemMap[currentTab].experimental.postMessage( JSON.stringify(read) );
                }
            }
            xhr.send();
        }
        readerMode = !readerMode;
    }

OrientationHelper { 
    id: orientationHelper

    Component {
        id: tabView
        WebView { 
            anchors.left: parent.left 
            anchors.top: parent.top
            anchors.fill: parent

            z: 2 // for drawer open/close control  
            anchors.topMargin: units.dp(43) //FIXME: should use navigator bar item

            MouseArea {
                id: contextOverlay;
                anchors.fill: parent;
                enabled: contextMenu.visible
                onClicked: contextMenu.visible = false
            }

            Rectangle{
                id: contextMenu
                visible: false
                width: units.dp(250); height: units.dp(230)
                x: units.dp(100); y: units.dp(50)
                color: "gray"
                radius: units.dp(5)
                Text {
                    id: contextUrl
                    color: "white"
                    font.pointSize: units.dp(8)
                    wrapMode: Text.WrapAnywhere
                    anchors {
                        top: parent.top; left: parent.left; right: parent.right;
                        margins: units.dp(20); topMargin: units.dp(10)
                    }
                }
                Column {
                    id: contextButtons
                    anchors.bottom: parent.bottom
                    anchors.bottomMargin: units.dp(10)
                    anchors.horizontalCenter: parent.horizontalCenter
                    spacing: units.dp(8)
                    ContextButton { 
                        label: "Open"; 
                        onClicked: { Tab.itemMap[currentTab].url = contextUrl.text; contextMenu.visible = false } 
                    }
                    ContextButton { 
                        label: "Open in New Tab"; 
                        onClicked: { bounce.start(); openNewTab("page"+salt(), contextUrl.text); contextMenu.visible = false } 
                    }
                    ContextButton { label: "Copy"; onClicked: { Clipboard.push(contextUrl.text); contextMenu.visible = false } }
                }
            }

            // FIXME: calculate scale, and position of screen
            function updateContextMenu(X, Y, url) {
                contextMenu.x = X; contextMenu.y = Y
                if (X + contextMenu.width/2 > root.width) {
                    contextMenu.x = root.width - contextMenu.width - units.dp(30)
                } else if (X - contextMenu.width/2 < 0) { 
                    contextMenu.x = units.dp(30)
                } else { 
                    contextMenu.x = X - contextMenu.width/2 - units.dp(10);
                }
                if (Y - contextMenu.height - units.dp(40) < 0) {
                    contextMenu.y = Y + units.dp(5)
                } else { 
                    contextMenu.y = Y - contextMenu.height - units.dp(20)
                }
                contextMenu.visible = true
                contextUrl.text = url
            }

            property real scale: experimental.test.contentsScale
            experimental.itemSelector: PopOver {}
            experimental.preferences.fullScreenEnabled: true;
            experimental.preferences.developerExtrasEnabled: true;

            // FIXME: use onDevicePixelRatioChanged when available
            //experimental.devicePixelRatio: 1.0;
            experimental.userScripts: [Qt.resolvedUrl("js/userscript.js")];
            experimental.preferences.navigatorQtObjectEnabled: true;
            experimental.onMessageReceived: {
                console.log('onMessageReceived: ' + message.data );
                var data = null
                try {
                    data = JSON.parse(message.data)
                } catch (error) {
                    console.log('onMessageReceived: ' + message.data );
                    return
                }
                switch (data.type) {
                    case 'link': {
                        if (data.target === '_blank') { // open link in new tab
                            bounce.start()
                            openNewTab('page-'+salt(), fixUrl(data.href))
                        }
                        break;
                    } 
		    case 'longpress': {
			updateContextMenu(data.pageX, data.pageY, fixUrl(data.href));
		    }
                }
            }
            experimental.userAgent: {
                if (mobileUserAgent && (root.width <= units.gu(60))) {
                    return "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3"
                } else if (mobileUserAgent) {
                    return "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3"
                } else { "" }
            }

            onLoadingChanged: { 
                contextMenu.visible = false;
                readerMode = false;
                urlText.text = Tab.itemMap[currentTab].url;
                if (loadRequest.status == WebView.LoadSucceededStatus) {
                    updateHistory(Tab.itemMap[currentTab].url, Tab.itemMap[currentTab].title, Tab.itemMap[currentTab].icon)
                }
            }
        }
    }

    Rectangle {
        id: drawer
        anchors.left: parent.left
        anchors.top: parent.top
        width: Tab.DrawerWidth
        height: parent.height
        color: "#33343E" 

        ListModel { id: tabModel }

        Component {
            id: tabDelegate
            Row {
                spacing: units.gu(1.2)
                Rectangle {
                    width: Tab.DrawerWidth
                    height: Tab.DrawerHeight
                    color: "transparent"
                    Image { 
                        height: units.gu(2); width: units.gu(2); 
                        source: hasTabOpen ? Tab.itemMap[model.pageid].icon : "icons/favicon.png";
                        anchors { top: parent.top; left: parent.left; margins: Tab.DrawerMargin; } 
                    }
                    Text { 
                        text: (typeof(Tab.itemMap[model.pageid]) !== "undefined" && Tab.itemMap[model.pageid].title !== "") ? 
                        Tab.itemMap[model.pageid].title : "Loading..";
                        color: "white"; 
                        font.pointSize: units.gu(1.4)
                        anchors { top: parent.top; left: parent.left; margins: Tab.DrawerMargin; 
                        leftMargin: Tab.DrawerMargin + units.gu(4); rightMargin: Tab.DrawerMargin + units.gu(1.5); right: parent.right } 
                        elide: Text.ElideRight 
                    }
                    MouseArea { 
                        anchors { top: parent.top; left: parent.left; bottom: parent.bottom; right: parent.right; rightMargin: units.gu(5)}
                        onClicked: { 
                            tabListView.currentIndex = index;
                            switchToTab(model.pageid);
                        }
                    }

                    Rectangle {
                        width: units.gu(5); height: units.gu(5) 
                        color: "transparent"
                        anchors { right: parent.right; top: parent.top}
                        Text {  // closeTab button
                            visible: tabListView.currentIndex === index
                            anchors { top: parent.top; right: parent.right; margins: Tab.DrawerMargin }
                            text: "\uF057"
                            font.family: fontAwesome.name
                            font.pointSize: units.gu(2)
                            color: "gray"
                        }
                        MouseArea { 
                            enabled: tabListView.currentIndex === index
                            anchors.fill: parent; 
                            onClicked: closeTab(model.index, model.pageid)
                        }
                    }
                }
            }
        }
        ListView {
            id: tabListView
            anchors.fill: parent

            // new tab button 
            header: Rectangle { 
                width: Tab.DrawerWidth
                height: Tab.DrawerHeight
                color: "transparent"
                Text { 
                    text: "\uF067"; font.family: fontAwesome.name; color: "white"; font.pointSize: units.gu(1.8) 
                    anchors { top: parent.top; left: parent.left; margins: Tab.DrawerMargin+units.gu(0.25); leftMargin: Tab.DrawerMargin+units.gu(1.25)}
                }
                Text { 
                    text: "<b>New Tab</b>"
                    color: "white"
                    font.pointSize: units.gu(1.8) 
                    anchors { top: parent.top; left: parent.left; margins: Tab.DrawerMargin; leftMargin: Tab.DrawerMargin+units.gu(3.75)}
                }
                MouseArea { 
                    anchors.fill: parent;
                    enabled: container.state == "opened" 
                    onClicked: {
                        openNewTab("page-"+salt(), Tab.HomePage); 
                    }
                }
            }

            model: tabModel
            delegate: tabDelegate 
            highlight: 

            Rectangle { 
                width: Tab.DrawerWidth; height: Tab.DrawerHeight 
                gradient: Gradient {
                    GradientStop { position: 0.1; color: "#1F1F23" }
                    GradientStop { position: 0.5; color: "#28282F" }
                    GradientStop { position: 0.8; color: "#2A2B31" }
                    GradientStop { position: 1.0; color: "#25252A" }

                }
            }
            highlightMoveDuration: 2
            highlightFollowsCurrentItem: true 
        }
    }

    Rectangle {
        id: container 
        anchors.left: parent.left 
        anchors.top: parent.top
        width: parent.width
        height: parent.height
        color: "#6B6C71" 
        z: 1 

        Rectangle { 
            id: navigatorBar
            height: units.dp(43); width: parent.width; anchors.top: parent.top; anchors.left: parent.left
            gradient: Gradient { 
                GradientStop { position: 0.0; color: "#ffffff" } 
                GradientStop { position: 1.0; color: "#eaeaea" } 
            }
        }
        // Navigator Bar, should use verticalCenter: parent.verticalCenter
        // drawer button 
        Item { 
            id: drawerButton
            width: units.gu(4); height: units.gu(4); anchors { top: parent.top; left: parent.left; margins: Tab.DrawerMargin; topMargin: units.gu(0.6) } 
            Text { 
                id: drawerButtonIcon
                text: (fontAwesome.status === FontLoader.Ready) ? "\uF0C9" : ""; 
                font { family: fontAwesome.name; pointSize: units.gu(3) } 
                color: "#AAAAAA" 
                style: Text.Sunken; styleColor: "gray"
            }
            MouseArea {
                anchors.fill: parent;
                anchors.margins: - units.gu(0.6); // trick to handle touch 
                onPressed: drawerButtonIcon.color = "#FED164";
                onClicked: { container.state == "closed" ? container.state = "opened" : container.state = "closed"; }
                onReleased: drawerButtonIcon.color = "#AAAAAA";
            }
        }

        Item {
            id: backButton
            width: units.gu(4); height: units.gu(4); anchors { top: parent.top; left: drawerButton.right; margins: Tab.DrawerMargin; topMargin: units.gu(0.6)}
            Text { 
                id: backButtonIcon
                text: "\uF053" 
                font { family: fontAwesome.name; pointSize: units.gu(3) }
                color: hasTabOpen ? (Tab.itemMap[currentTab].canGoBack ? "#AAAAAA" : "lightgray") : "lightgray"
                style: Text.Sunken; styleColor: "gray"
            }
            MouseArea { 
                anchors.fill: parent; anchors.margins: - units.gu(0.6); 
                onPressed: backButtonIcon.color = "#FED164";
                onClicked: { if (hasTabOpen && Tab.itemMap[currentTab].canGoBack) Tab.itemMap[currentTab].goBack()  }
                onReleased: backButtonIcon.color = "#AAAAAA";
            }
        }
        // TODO: forward button? \uF061 

        Rectangle { 
            id: urlBar 
            anchors { left: backButton.right; top: parent.top; right: exportButton.left; margins: units.gu(0.8); rightMargin: units.gu(1.8); topMargin: units.gu(1)} 
            color: "white"
            height: units.gu(3.3)
            border { width: units.gu(0.12); color: "black" }
            radius: units.gu(0.625) 
            width: parent.width - units.gu(7.5)

            Rectangle {
                anchors { top: parent.top; bottom: parent.bottom; left: parent.left }
                radius: units.gu(0.375)

                width: hasTabOpen ?  parent.width / 100 * Math.max(5, Tab.itemMap[currentTab].loadProgress) : 0 
                color: "#FED164" // light yellow 
                opacity: 0.4
                visible: hasTabOpen ? Tab.itemMap[currentTab].loading : false 
            }

            TextField {
                id: urlText
                hasClearButton: false
                text: hasTabOpen ? Tab.itemMap[currentTab].url : ""
                anchors { fill: parent; margins: units.dp(2); }
                onAccepted: { 
                    loadUrl(urlText.text)
                    urlText.text = urlText.text;
                }
                //    urlDisplayText.text = historyListView.model.get(historyListView.currentIndex).url.replace( /(<([^>]+)>)/ig, "")
                onActiveFocusChanged: { 
                    // FIXME: use State to change property  
                    if (urlText.activeFocus) { 
                        urlText.selectAll(); parent.border.color = "#2E6FFD"; parent.border.width = units.gu(0.25);
                    } else { 
                        parent.border.color = "black"; parent.border.width = units.gu(0.12); 
                    }
                }
                onTextChanged: {
                    if (urlText.activeFocus && urlText.text !== "") {
                        queryHistory(urlText.text)
                    } else { historyModel.clear() }
                }
            }            

            Text {
                id: stopButton
                anchors { right: urlBar.right; rightMargin: units.gu(0.6); verticalCenter: parent.verticalCenter}
                text: "\uF00D"
                font { family: fontAwesome.name; pointSize: units.gu(1.8) }
                color: "gray"
                visible: ( hasTabOpen && Tab.itemMap[currentTab].loadProgress < 100 && !urlText.focus) ? 
                true : false
                MouseArea {
                    anchors { fill: parent; margins: - units.gu(1.25); }
                    onClicked: { Tab.itemMap[currentTab].stop(); }
                }
            }
            Text {
                id: reloadButton
                anchors { right: urlBar.right; rightMargin: units.gu(0.6); verticalCenter: parent.verticalCenter}
                text: "\uF01E"
                font { family: fontAwesome.name; pointSize: units.gu(1.8) }
                color: "gray"
                visible: ( hasTabOpen && Tab.itemMap[currentTab].loadProgress == 100 && !urlText.focus ) ? 
                true : false 
                MouseArea {
                    anchors { fill: parent; margins: - units.gu(1.25); }
                    onClicked: { Tab.itemMap[currentTab].reload(); }
                }
            }
            Text {
                id: clearButton
                anchors { right: urlBar.right; rightMargin: units.gu(0.6); verticalCenter: parent.verticalCenter}
                text: "\uF057"
                font { family: fontAwesome.name; pointSize: units.gu(1.8) }
                color: "gray"
                visible: urlText.focus
                MouseArea {
                    anchors { fill: parent; margins: - units.gu(1.25); }
                    onClicked: { urlText.text = ""; }
                }
            }
        }

        Item {
            id: exportButton
            width: units.gu(4); height: units.gu(4); anchors { top: parent.top; right: parent.right; margins: Tab.DrawerMargin; topMargin: units.gu(0.8); rightMargin: units.dp(15)}
            Text { 
                text: "\uF013"
                font { family: fontAwesome.name; pointSize: units.gu(3) }
                color: (readerMode && hasTabOpen) ? "#FED164" : "#AAAAAA"
                style: Text.Sunken; styleColor: "gray"
            }
            MouseArea { 
                anchors.fill: parent; anchors.margins: units.dp(-5); 
                onClicked: PopupUtils.open(actionSelectionPopover, exportButton);
            }
        }

        Component {
            id: actionSelectionPopover

            ActionSelectionPopover {

                Standard {
                    control: Switch {
                        anchors.verticalCenter: parent.verticalCenter
                        checked: mobileUserAgent 
                        onClicked: { 
                            mobileUserAgent = !mobileUserAgent;
                            if (hasTabOpen) Tab.itemMap[currentTab].reload();
                        }
                    }
                }
                anchors.rightMargin: units.dp(-10);
                actions: ActionList {
                    Action {
                        text: i18n.tr("Mobile Layout")
                    }
                    Action {
                        text: readerMode ? i18n.tr("Disable Reader Mode") : i18n.tr("Enable Reader Mode")
                        onTriggered: if (hasTabOpen) toggleReaderMode(); 
                    }
                    Action {
                        text: i18n.tr("Share Link..")
                        onTriggered: print(text)
                    }
                } 
            }
        }

        Item { 
            id: suggestionContainer
            width: navigatorBar.width - units.dp(50)
            height: suggestionDialog.height + units.gu(3) + units.gu(4) 
            anchors { top: parent.top; topMargin: units.dp(20); left: parent.left; leftMargin: units.dp(30); }
            visible: (urlText.focus && historyModel.count > 0)
            z: 5

            Rectangle {
                id: suggestionDialog
                color: "lightgray"
                radius: units.gu(0.625) 
                anchors.centerIn: parent 
                width: navigatorBar.width - units.dp(65) 
                height: (historyModel.count > 3) ? ((historyModel.count <= 6) ? historyModel.count * units.gu(5) : units.gu(35)) : units.gu(10)
                anchors { top: parent.top; topMargin: units.gu(6.25); left: parent.left; leftMargin: units.dp(30); }

                Text { // caret-up 
                    anchors.top: parent.top
                    anchors.topMargin: - units.gu(4.8)
                    anchors.left: parent.horizontalCenter
                    anchors.leftMargin: - units.gu(3.75)
                    font { family: fontAwesome.name; pointSize: units.gu(6.6) }
                    text: "\uF0D8"; 
                    color: "lightgray" 
                }

                ListView { 
                    id: historyListView
                    anchors.fill: parent
                    anchors.topMargin: units.gu(3.75) 
                    anchors.bottomMargin: units.gu(3.75)
                    model: historyModel 
                    delegate: historyDelegate
                    ListModel { 
                        id: historyModel
                    }
                    Component {
                        id: historyDelegate
                        Rectangle { 
                            color: "transparent"
                            height: Tab.DrawerHeight
                            width: parent.width 
                            Text { 
                                anchors {
                                    top: parent.top; left: parent.left; right: parent.right
                                    margins: units.dp(8); leftMargin: units.dp(10);
                                }
                                text: '<b>'+ model.title +'<b>'
                                font.pointSize: units.dp(12)
                                elide: Text.ElideRight
                            }
                            Text { 
                                anchors { 
                                    top: parent.top; left: parent.left; right: parent.right
                                    margins: units.dp(8);
                                    topMargin: units.dp(30); leftMargin: units.dp(10);
                                }
                                color: "#565051" // darkgray
                                text: model.url
                                font.pointSize: units.dp(8)
                                elide: Text.ElideMiddle
                            }
                            MouseArea { 
                                anchors.fill: parent; 
                                onClicked: loadUrl(model.url) 
                            }
                        }
                    }
                } // end of historyListView
            }

        }

        DropShadow {
            id: suggestionShadow;                
            z: 5
            visible: (urlText.focus && historyModel.count > 0)
            anchors.fill: source
            cached: true;                          
            horizontalOffset: units.gu(0.38);
            verticalOffset: units.gu(0.38);
            radius: units.gu(1.5);
            samples: units.gu(2);
            color: "#80000000";
            smooth: true;
            source: suggestionContainer;
        }

        MouseArea { 
            z: (container.state == "opened") ? 3 : 1
            anchors.fill: parent
            anchors.topMargin: units.dp(43) 
            onClicked: { container.state == "closed" ? container.state = "opened" : container.state = "closed"; }
        }
        states: [
            State{
                name: "opened"
                PropertyChanges { target: container; anchors.leftMargin: Tab.DrawerWidth }
            },
            State {
                name: "closed"
                PropertyChanges { target: container; anchors.leftMargin: 0 }
            }
        ]
        transitions: [
            Transition {
                to: "*"
                NumberAnimation { target: container; properties: "anchors.leftMargin"; duration: 300; easing.type: Easing.InOutQuad; }
            }
        ]
    }
} // end of orientationHelper
}
